/*
 *  Copyright (C) 2009 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#include "kiba-info-win-manager.h"

G_DEFINE_TYPE (KibaInfoWinManager, kiba_info_win_manager, G_TYPE_OBJECT)

static KibaInfoWinManager* manager_singleton = NULL;

enum {
	SIG_WIN_ADDED = 0,
	SIG_WIN_REMOVED,
	SIG_MANAGER_LAST
};
static guint manager_signals[SIG_MANAGER_LAST] = { 0 };

/// private methods

static void
destroyed_cb(GtkWidget *widget,
		gpointer data)
{
	KibaInfoWinManager* manager = data;
	g_signal_emit (manager,
		manager_signals[SIG_WIN_REMOVED], 0, widget);
}

/// public methods

void
kiba_info_win_manager_add_window(KibaInfoWinManager* manager,
		KibaInfoWin *info_win)
{
	g_signal_emit (manager,
		manager_signals[SIG_WIN_ADDED], 0, info_win);
	g_signal_connect(info_win, "destroy",
		G_CALLBACK(destroyed_cb), manager);
}

/// GObject implementation

/// Class Functions

static void
kiba_info_win_manager_class_init (KibaInfoWinManagerClass *class)
{
	/**
	 * KibaInfoWinManager::info-win-added:
	 * @info_win_manager: the object on which the signal is emitted
	 * @info_win: the win that was added.
	 *
	 * The info-win-added signal is emitted when a new win was added.
	 */
	manager_signals[SIG_WIN_ADDED] =
		g_signal_new ("info-win-added",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET (KibaInfoWinManagerClass, info_win_added),
			NULL, NULL,
			g_cclosure_marshal_VOID__OBJECT,
			G_TYPE_NONE, 1, G_TYPE_OBJECT);
	/**
	 * KibaInfoWinManager::info-win-removed:
	 * @info_win_manager: the object on which the signal is emitted
	 * @info_win: the win that was removed.
	 *
	 * The info-win-removed signal is emitted when a new win was removed.
	 */
	manager_signals[SIG_WIN_REMOVED] =
		g_signal_new ("info-win-removed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET (KibaInfoWinManagerClass, info_win_removed),
			NULL, NULL,
			g_cclosure_marshal_VOID__OBJECT,
			G_TYPE_NONE, 1, G_TYPE_OBJECT);
}

static void
kiba_info_win_manager_init (KibaInfoWinManager *info_win_manager)
{
}

static KibaInfoWinManager*
kiba_info_win_manager_new()
{
	return g_object_new (KIBA_TYPE_INFO_WIN_MANAGER, NULL);
}

KibaInfoWinManager*
kiba_info_win_manager_get()
{
	return manager_singleton ? manager_singleton :
		(manager_singleton = kiba_info_win_manager_new());
}
